import pickle
from pathlib import Path
from typing import Any, Union

from .path_handler import create_path


def read_pickle(path: Union[str, Path]) -> Any:
    with open(path, "rb") as f:
        x = pickle.load(f)
    return x


def write_pickle(x, path: Union[str, Path]) -> None:
    path = create_path(path)
    path.parent.mkdir(parents=True, exist_ok=True)

    with open(path, "wb") as f:
        pickle.dump(x, f)



from pathlib import Path
from typing import Any, Union

import lz4.frame


def read_lz4(path: str) -> bytes:
    with lz4.frame.open(path, mode="rb") as f:
        x = lz4.frame.decompress(f.read())

    return x


def write_lz4(x: bytes, path: Union[str, Path]) -> None:
    path = Path(path)
    path.parent.mkdir(parents=True, exist_ok=True)

    with lz4.frame.open(path, mode="wb") as f:
        f.write(lz4.frame.compress(x, compression_level=16))



from pathlib import Path
from typing import Any, Union

import numpy as np

from .path_handler import create_path


def read_numpy(path: Union[str, Path]) -> np.ndarray:
    with open(path, "rb") as f:
        x = np.load(file=f)
    return x


def write_numpy(x: np.ndarray, path: Union[str, Path]) -> None:
    path = create_path(path)
    path.parent.mkdir(parents=True, exist_ok=True)
    
    with open(path, "wb") as f:
        np.save(file=f, arr=x)



__all__ = [
    "join_path",
    #
    "read",
    "read_csv",
    "read_gzip",
    "read_gzip_list",
    "read_gzip_json_list",
    "read_json",
    "read_jsonl",
    "read_list",
    "read_lz4",
    "read_numpy",
    "read_pickle",
    #
    "load",
    "load_csv",
    "load_gzip",
    "load_gzip_list",
    "load_gzip_json_list",
    "load_json",
    "load_jsonl",
    "load_list",
    "write_lz4",
    "load_numpy",
    "load_pickle",
    #
    "write",
    "write_csv",
    "write_gzip",
    "write_gzip_list",
    "write_gzip_json_list",
    "write_json",
    "write_jsonl",
    "write_list",
    "write_lz4",
    "write_numpy",
    "write_pickle",
    #
    "save",
    "save_csv",
    "save_gzip",
    "save_gzip_list",
    "save_gzip_json_list",
    "save_json",
    "save_jsonl",
    "save_list",
    "save_lz4",
    "save_numpy",
    "save_pickle",
]

from .handlers.csv_handler import read_csv, write_csv
from .handlers.gzip_handler import (
    read_gzip,
    read_gzip_json_list,
    read_gzip_list,
    write_gzip,
    write_gzip_json_list,
    write_gzip_list,
)
from .handlers.json_handler import read_json, write_json
from .handlers.jsonl_handler import read_jsonl, write_jsonl
from .handlers.list_handler import read_list, write_list
from .handlers.lz4_handler import read_lz4, write_lz4
from .handlers.numpy_handler import read_numpy, write_numpy
from .handlers.path_handler import join_path
from .handlers.pickle_handler import read_pickle, write_pickle
from .handlers.string_handler import read, write

# Aliases ======================================================================
# Read / Load ------------------------------------------------------------------
load = read
load_csv = read_csv
load_gzip = read_gzip
load_gzip_list = read_gzip_list
load_gzip_json_list = read_gzip_json_list
load_json = read_json
load_jsonl = read_jsonl
load_list = read_list
load_lz4 = read_lz4
load_numpy = read_numpy
load_pickle = read_pickle
# Write / Save -----------------------------------------------------------------
save = write
save_csv = write_csv
save_gzip = write_gzip
save_gzip_list = write_gzip_list
save_gzip_json_list = write_gzip_json_list
save_json = write_json
save_jsonl = write_jsonl
save_list = write_list
save_lz4 = write_lz4
save_numpy = write_numpy
save_pickle = write_pickle



import csv
import sys
from pathlib import Path
from typing import Callable, Dict, List, Union

from .common import apply_callback
from .path_handler import create_path


def read_csv_to_generator(
    path: Union[str, Path],
    delimiter=",",
    callback: Callable = None,
    encoding: str = "utf-8",
):
    with open(path, "r", encoding=encoding) as f:
        lines = csv.DictReader(f, skipinitialspace=True, delimiter=delimiter)
        for line in lines:
            yield apply_callback(line, callback)


def read_csv_to_list(
    path: Union[str, Path],
    delimiter=",",
    callback: Callable = None,
    encoding: str = "utf-8",
):
    with open(path, "r", encoding=encoding) as f:
        x = list(csv.DictReader(f, skipinitialspace=True, delimiter=delimiter))
    return [apply_callback(y, callback) for y in x]


def read_csv(
    path: Union[str, Path],
    callback: Callable = None,
    delimiter=",",
    generator: bool = False,
    encoding: str = "utf-8",
) -> List[Dict]:
    csv.field_size_limit(sys.maxsize)
    if generator:
        return read_csv_to_generator(path, delimiter, callback, encoding)
    else:
        return read_csv_to_list(path, delimiter, callback, encoding)


def write_csv(
    x: List[Dict],
    path: Union[str, Path],
    callback: Callable = None,
    delimiter: str = ",",
    encoding: str = "utf-8",
):
    path = create_path(path)
    path.parent.mkdir(parents=True, exist_ok=True)

    with open(path, "w", encoding=encoding, newline="") as f:
        keys = list(x[0])
        dict_writer = csv.DictWriter(f, keys, delimiter=delimiter)
        dict_writer.writeheader()
        dict_writer.writerows([apply_callback(y, callback) for y in x])



from pathlib import Path
from typing import Callable, Dict, List, Union

import orjson

from .common import apply_callback
from .path_handler import create_path


def read_jsonl_to_generator(path: Union[str, Path], callback: Callable = None):
    with open(path, "rb") as f:
        for line in f:
            line = orjson.loads(line)
            yield apply_callback(line, callback)


def read_jsonl_to_list(path: Union[str, Path], callback: Callable = None):
    with open(path, "rb") as f:
        lines = [orjson.loads(line) for line in f]
        return [apply_callback(line, callback) for line in lines]


def read_jsonl(
    path: Union[str, Path],
    callback: Callable = None,
    generator: bool = False,
) -> List[Dict]:
    if generator:
        return read_jsonl_to_generator(path, callback)
    else:
        return read_jsonl_to_list(path, callback)


def write_jsonl(
    x: List[dict],
    path: Union[str, Path],
    callback: Callable = None,
) -> None:
    path = create_path(path)
    path.parent.mkdir(parents=True, exist_ok=True)

    with open(path, "wb") as f:
        for y in x:
            f.write(orjson.dumps(apply_callback(y, callback)) + "\n".encode())



from pathlib import Path
from typing import Union

from .path_handler import create_path


def read(path: Union[str, Path], encoding: str = "utf-8") -> str:
    with open(path, "r", encoding=encoding) as f:
        x = f.read()
    return x


def write(x: str, path: Union[str, Path], encoding: str = "utf-8") -> None:
    path = create_path(path)
    path.parent.mkdir(parents=True, exist_ok=True)

    with open(path, "w", encoding=encoding) as f:
        f.write(x)



from pathlib import Path
from typing import Dict, Union

import orjson

from .path_handler import create_path


def read_json(path: Union[str, Path]) -> Dict:
    with open(path, "rb") as f:
        x = orjson.loads(f.read())
    return x


def write_json(x: Dict, path: Union[str, Path]) -> None:
    path = create_path(path)
    path.parent.mkdir(parents=True, exist_ok=True)

    with open(path, "wb") as f:
        f.write(orjson.dumps(x, option=orjson.OPT_INDENT_2))



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="oneliner-utils",
    version="0.1.0",
    author="Elias Bassani",
    author_email="elias.bssn@gmail.com",
    description="Oneline Utils for Python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/AmenRa/oneliner-utils",
    packages=setuptools.find_packages(),
    install_requires=["numpy", "orjson", "lz4"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Intended Audience :: Science/Research",
        "Operating System :: OS Independent",
        "Topic :: Text Processing :: General",
    ],
    keywords=["oneliner", "oneliners", "utils", "utilities"],
    python_requires=">=3.7",
)



import gzip
from pathlib import Path
from typing import Any, Callable, List, Union

import orjson

from .common import apply_callback
from .path_handler import create_path


# String -----------------------------------------------------------------------
def read_gzip(path: Union[str, Path]) -> Any:
    with gzip.open(path, "rt") as f:
        x = f.read()
    return x


def write_gzip(x: str, path: Union[str, Path]) -> Any:
    path = create_path(path)
    path.parent.mkdir(parents=True, exist_ok=True)

    with gzip.open(path, "wt") as f:
        f.write(x)


# List of Strings --------------------------------------------------------------
def read_gzip_to_generator(path: Union[str, Path], callback: Callable = None):
    with gzip.open(path, "rt") as f:
        for line in f:
            line = line.strip()
            yield apply_callback(line, callback)


def read_gzip_to_list(path: Union[str, Path], callback: Callable = None):
    with gzip.open(path, "rt") as f:
        lines = f.read().splitlines()
        return [apply_callback(line, callback) for line in lines]


def read_gzip_list(
    path: Union[str, Path],
    callback: Callable = None,
    generator: bool = False,
) -> List:
    if generator:
        return read_gzip_to_generator(path, callback)
    else:
        return read_gzip_to_list(path, callback)


def write_gzip_list(
    x: List[str],
    path: Union[str, Path],
    callback: Callable = None,
) -> List:
    path = create_path(path)
    path.parent.mkdir(parents=True, exist_ok=True)

    with gzip.open(path, "wt") as f:
        for y in x:
            f.write(apply_callback(y, callback) + "\n")


# List of Json -----------------------------------------------------------------
def read_gzip_to_json_generator(
    path: Union[str, Path], callback: Callable = None
):
    with gzip.open(path, "rb") as f:
        for line in f:
            line = orjson.loads(line)
            yield apply_callback(line, callback)


def read_gzip_to_json_list(path: Union[str, Path], callback: Callable = None):
    with gzip.open(path, "rb") as f:
        lines = [orjson.loads(line) for line in f]
        return [apply_callback(line, callback) for line in lines]


def read_gzip_json_list(
    path: Union[str, Path],
    callback: Callable = None,
    generator: bool = False,
) -> List:
    if generator:
        return read_gzip_to_json_generator(path, callback)
    else:
        return read_gzip_to_json_list(path, callback)


def write_gzip_json_list(
    x: List[str],
    path: Union[str, Path],
    callback: Callable = None,
) -> List:
    path = create_path(path)
    path.parent.mkdir(parents=True, exist_ok=True)

    with gzip.open(path, "wb") as f:
        for y in x:
            f.write(orjson.dumps(apply_callback(y, callback)) + "\n".encode())



import os
from pathlib import Path


def create_path(path: str) -> Path:
    path = Path(path)
    path.parent.mkdir(parents=True, exist_ok=True)

    return path


def join_path(*args: str) -> str:
    return Path(os.path.join(*args))



from pathlib import Path
from typing import Callable, List, Union

from .common import apply_callback
from .path_handler import create_path


def read_file_to_generator(
    path: Union[str, Path],
    callback: Callable = None,
    encoding: str = "utf-8",
):
    with open(path, "r", encoding=encoding) as f:
        for line in f:
            line = line.strip()
            yield callback(line) if callback is not None else line


def read_file_to_list(
    path: Union[str, Path],
    callback: Callable = None,
    encoding: str = "utf-8",
):
    with open(path, "r", encoding=encoding) as f:
        lines = f.read().splitlines()
        lines = [apply_callback(line, callback) for line in lines]
        return lines


def read_list(
    path: Union[str, Path],
    callback: Callable = None,
    generator: bool = False,
    encoding: str = "utf-8",
) -> List[str]:
    if generator:
        return read_file_to_generator(path, callback, encoding)
    else:
        return read_file_to_list(path, callback, encoding)


def write_list(
    x: List[str],
    path: Union[str, Path],
    callback: Callable = None,
    encoding: str = "utf-8",
) -> None:
    path = create_path(path)
    path.parent.mkdir(parents=True, exist_ok=True)

    with open(path, "w", encoding=encoding) as f:
        for y in x:
            f.write(apply_callback(y, callback) + "\n")
